import code


class Console(code.InteractiveConsole):
    """
    Interactive console providing a sandboxed namespace for executing code
    examples.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = "Internals"

    ### INITIALIZER ###

    def __init__(self, client=None, namespace=None):
        namespace = namespace or {}
        namespace["__builtins__"] = __builtins__.copy()
        namespace["__name__"] = "__main__"
        namespace["__package__"] = None
        code.InteractiveConsole.__init__(self, locals=namespace, filename="<stdin>")
        self.client = client

    ### PUBLIC METHODS ###

    def showsyntaxerror(self, filename=None):
        r"""Proxies Python's InteractiveConsole.showsyntaxerror().
        """
        code.InteractiveConsole.showsyntaxerror(self, filename=filename)
        self.client.register_error()

    def showtraceback(self):
        r"""Proxies Python's InteractiveConsole.showtraceback().
        """
        code.InteractiveConsole.showtraceback(self)
        self.client.register_error()

    def unregister_error(self):
        r"""Unregisters the last error registered in the current document
        handler.

        This occurs when the interpreting code block permits errors to appear.
        """
        if self.client:
            self.client.unregister_error()

    ### PUBLIC PROPERTIES ###

    @property
    def errored(self):
        r"""Is true if the last line executed by the console errored.
        """
        if self.client:
            return self.client.errored
        return False



import enum
import inspect
from typing import cast

from uqbar.apis.MemberDocumenter import MemberDocumenter


class ClassDocumenter(MemberDocumenter):
    """
    A basic class documenter.

    ::

        >>> import uqbar.apis
        >>> path = 'uqbar.apis.ClassDocumenter.ClassDocumenter'
        >>> documenter = uqbar.apis.ClassDocumenter(path)
        >>> documentation = str(documenter)
        >>> print(documentation)
        .. autoclass:: ClassDocumenter
           :members:
           :undoc-members:

    .. tip::

       Subclass :py:class:`~uqbar.apis.ClassDocumenter` to implement your own
       custom class documentation output. You'll need to provide your desired
       reStructuredText output via an overridden
       :py:meth:`~uqbar.apis.ClassDocumenter.ClassDocumenter.__str__`
       implementation.

       See :py:class:`~uqbar.apis.SummarizingClassDocumenter` for an example.

    :param package_path: the module path and name of the member to document
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = "Documenters"

    ### SPECIAL METHODS ###

    def __str__(self) -> str:
        name = getattr(self.client, "__name__")
        if issubclass(self.client, Exception):  # type: ignore
            return ".. autoexception:: {}".format(name)
        return "\n".join(
            [".. autoclass:: {}".format(name), "   :members:", "   :undoc-members:"]
        )

    ### PUBLIC METHODS ###

    @classmethod
    def validate_client(cls, client: object, module_path: str) -> bool:
        return isinstance(client, type) and getattr(client, "__module__") == module_path

    ### PUBLIC PROPERTIES ###

    @property
    def documentation_section(self) -> str:
        client = cast(type, self.client)
        if hasattr(client, "__documentation_section__"):
            section = getattr(client, "__documentation_section__")
            if section is not None:
                return section
        if inspect.isabstract(client):
            return "Abstract Classes"
        elif issubclass(client, enum.Enum):
            return "Enumerations"
        elif issubclass(client, Exception):
            return "Exceptions"
        return "Classes"



"""
Tools for auto-generating API documention.
"""
import importlib
import pathlib

from .APIBuilder import APIBuilder  # noqa
from .ClassDocumenter import ClassDocumenter  # noqa
from .FunctionDocumenter import FunctionDocumenter  # noqa
from .InheritanceGraph import InheritanceGraph  # noqa
from .MemberDocumenter import MemberDocumenter  # noqa
from .ModuleDocumenter import ModuleDocumenter  # noqa
from .ModuleNode import ModuleNode  # noqa
from .PackageNode import PackageNode  # noqa
from .RootDocumenter import RootDocumenter  # noqa
from .SummarizingClassDocumenter import SummarizingClassDocumenter  # noqa
from .SummarizingModuleDocumenter import SummarizingModuleDocumenter  # noqa
from .SummarizingRootDocumenter import SummarizingRootDocumenter  # noqa


def collect_source_paths(source_paths, recurse_subpackages=True):
    visited_paths = set()
    path_stack = []
    for source_path in source_paths:
        source_path = pathlib.Path(source_path).resolve().absolute()
        assert source_path.exists()
        path_stack.append(source_path)
    while path_stack:
        current_path = path_stack.pop()
        if current_path in visited_paths:
            continue
        if current_path.is_dir():
            if not (current_path / "__init__.py").exists():
                continue
            if recurse_subpackages:
                for path in current_path.iterdir():
                    if path.is_dir() and (path / "__init__.py").exists():
                        path_stack.append(path)
                    elif path.suffix in (".py", ".pyx"):
                        visited_paths.add(path)
            else:
                visited_paths.add(current_path / "__init__.py")
                for path in current_path.iterdir():
                    if path.is_dir():
                        continue
                    # Permit nominative modules
                    if path.suffix in (".py", ".pyx"):
                        package_path = source_path_to_package_path(path)
                        module = importlib.import_module(package_path)
                        if hasattr(module, path.with_suffix("").name):
                            visited_paths.add(path)
        elif current_path.suffix in (".py", ".pyx"):
            visited_paths.add(current_path)
    return sorted(visited_paths)


def source_path_to_package_path(path):
    path = pathlib.Path(path)
    if not (path.parent / "__init__.py").exists():
        return path.with_suffix("").name
    root = path
    while (root.parent / "__init__.py").exists():
        root = root.parent
    path = path.with_suffix("")
    if path == root:
        return path.name
    parts = (root.name,) + path.relative_to(root).parts
    if parts[-1] == "__init__":
        parts = parts[:-1]
    return ".".join(parts)



#!/usr/bin/env python
import pathlib

import setuptools

package_name = "uqbar"


def read_version():
    root_path = pathlib.Path(__file__).parent
    version_path = root_path / package_name / "_version.py"
    with version_path.open() as file_pointer:
        file_contents = file_pointer.read()
    local_dict = {}
    exec(file_contents, None, local_dict)
    return local_dict["__version__"]


description = "Tools for building documentation with Sphinx, Graphviz and LaTeX"

classifiers = [
    "Development Status :: 3 - Alpha",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Natural Language :: English",
    "Operating System :: MacOS",
    "Operating System :: POSIX",
    "Programming Language :: Python :: 3.6",
]

install_requires = [
    "Sphinx>=1.7.0",
    "Unidecode>=0.4.21",
    "sphinx-autodoc-typehints>=1.3.0",
    "sphinx-rtd-theme>=0.4.0",
]

extras_require = {
    "test": [
        "black",
        "flake8",
        "isort",
        "mypy >= 0.660",
        "pytest >= 4.1.0",
        "pytest-cov >= 2.6.0",
    ]
}

keywords = ["sphinx", "graphviz", "latex", "documentation"]

with open("README.md", "r") as file_pointer:
    long_description = file_pointer.read()

version = read_version()

if __name__ == "__main__":
    setuptools.setup(
        author="Josiah Wolf Oberholtzer",
        author_email="josiah.oberholtzer@gmail.com",
        classifiers=classifiers,
        description=description,
        extras_require=extras_require,
        include_package_data=True,
        install_requires=install_requires,
        keywords=keywords,
        long_description=description,
        name=package_name,
        packages=[package_name],
        url="https://github.com/josiah-wolf-oberholtzer/uqbar",
        version=version,
        zip_safe=False,
    )



from typing import Tuple  # noqa

from uqbar.containers import UniqueTreeContainer


class RecordGroup(UniqueTreeContainer):
    """ A Graphviz record field group.

    ::

        >>> import uqbar.graphs
        >>> group = uqbar.graphs.RecordGroup()
        >>> group.extend([
        ...     uqbar.graphs.RecordField(),
        ...     uqbar.graphs.RecordGroup([
        ...         uqbar.graphs.RecordField(),
        ...         uqbar.graphs.RecordField(),
        ...         ]),
        ...     uqbar.graphs.RecordField(),
        ...     ])
        >>> print(format(group, 'graphviz'))
        { <f_0> | { <f_0> | <f_0> } | <f_0> }

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = "Record Field Classes"

    ### INITIALIZER ###

    def __init__(self, children=None, *, name: str = None) -> None:
        UniqueTreeContainer.__init__(self, name=name, children=children)

    ### SPECIAL METHODS ###

    def __format__(self, format_spec=None) -> str:
        # TODO: make the format specification options machine-readable
        if format_spec == "graphviz":
            return self.__format_graphviz__()
        return str(self)

    def __format_graphviz__(self) -> str:
        result = " | ".join(_ for _ in (format(_, "graphviz") for _ in self) if _)
        if result:
            result = "{{ {} }}".format(result)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _node_class(self) -> Tuple[type, ...]:
        import uqbar.graphs

        return (uqbar.graphs.RecordField, uqbar.graphs.RecordGroup)
